@@ -458,7 +458,7 @@ def command_prompt(pymux, cli, variables):
         client_state.command_mode = True
 
     # Go to insert mode.
-    client_state.vi_state.input_mode = InputMode.INSERT
+    cli.vi_state.input_mode = InputMode.INSERT
 
 
 @cmd('send-prefix')
@@ -525,11 +525,14 @@ def send_keys(pymux, cli, variables):
             pane.process.write_key(k)
 
 
-@cmd('copy-mode')
+@cmd('copy-mode', options='[-u]')
 def copy_mode(pymux, cli, variables):
     """
     Enter copy mode.
     """
+    go_up = variables['-u']  # Go in copy mode and page-up directly.
+    # TODO: handle '-u'
+
     pane = pymux.arrangement.get_active_pane(cli)
     pane.enter_copy_mode()
 
