@@ -4,13 +4,13 @@ from prompt_toolkit.application import Application
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit.buffer import Buffer, AcceptAction
 from prompt_toolkit.buffer_mapping import BufferMapping
-from prompt_toolkit.enums import DUMMY_BUFFER
+from prompt_toolkit.enums import DUMMY_BUFFER, EditingMode
 from prompt_toolkit.eventloop.callbacks import EventLoopCallbacks
 from prompt_toolkit.eventloop.posix import PosixEventLoop
 from prompt_toolkit.filters import Condition
 from prompt_toolkit.input import PipeInput
 from prompt_toolkit.interface import CommandLineInterface
-from prompt_toolkit.key_binding.vi_state import InputMode, ViState
+from prompt_toolkit.key_binding.vi_state import InputMode
 from prompt_toolkit.layout.screen import Size
 from prompt_toolkit.terminal.vt100_output import Vt100_Output, _get_size
 
@@ -33,6 +33,8 @@ import signal
 import six
 import sys
 import tempfile
+import threading
+import time
 import traceback
 import weakref
 
@@ -52,9 +54,6 @@ class ClientState(object):
         #: Error/info message.
         self.message = None
 
-        # Vi state. (Each client has its own state.)
-        self.vi_state = ViState()
-
         # True when the command prompt is visible.
         self.command_mode = False
 
@@ -93,11 +92,13 @@ class Pymux(object):
         # Options
         self.enable_mouse_support = True
         self.enable_status = True
+        self.enable_pane_status = False
         self.enable_bell = True
         self.remain_on_exit = False
         self.status_keys_vi_mode = False
         self.mode_keys_vi_mode = False
         self.history_limit = 2000
+        self.status_interval = 4
         self.default_terminal = 'xterm-256color'
         self.status_left = '[#S] '
         self.status_left_length = 20
@@ -141,6 +142,20 @@ class Pymux(object):
 
         self.style = PymuxStyle()
 
+    def _start_auto_refresh_thread(self):
+        """
+        Start the background thread that auto refreshes all clients according to
+        `self.status_interval`.
+        """
+        def run():
+            while True:
+                time.sleep(self.status_interval)
+                self.invalidate()
+
+        t = threading.Thread(target=run)
+        t.daemon = True
+        t.start()
+
     def get_client_state(self, cli):
         """
         Return the ClientState instance for this CommandLineInterface.
@@ -368,7 +383,7 @@ class Pymux(object):
             """ When the focus changes to a read/write buffer, make sure to go
             to insert mode. This happens when the ViState was set to NAVIGATION
             in the copy buffer. """
-            vi_state = self.key_bindings_manager.pt_key_bindings_manager.get_vi_state(cli)
+            vi_state = cli.vi_state
 
             if cli.current_buffer.read_only():
                 vi_state.input_mode = InputMode.NAVIGATION
@@ -382,7 +397,8 @@ class Pymux(object):
             mouse_support=Condition(lambda cli: self.enable_mouse_support),
             use_alternate_screen=True,
             style=self.style,
-            get_title=get_title)
+            get_title=get_title,
+            on_invalidate=(lambda cli: self.invalidate()))
 
         cli = CommandLineInterface(
             application=application,
@@ -390,6 +406,23 @@ class Pymux(object):
             input=input,
             eventloop=self.eventloop)
 
+        # Synchronize the Vi state with the CLI object.
+        # (This is stored in the current class, but expected to be in the
+        # CommandLineInterface.)
+        def sync_vi_state(_):
+            client_state = self.get_client_state(cli)
+            VI = EditingMode.VI
+            EMACS = EditingMode.EMACS
+
+            if (client_state.confirm_text or client_state.prompt_command or
+                    client_state.command_mode):
+                cli.editing_mode = VI if self.status_keys_vi_mode else EMACS
+            else:
+                cli.editing_mode = VI if self.mode_keys_vi_mode else EMACS
+
+        cli.input_processor.beforeKeyPress += sync_vi_state
+        cli.input_processor.afterKeyPress += sync_vi_state
+
         # Set render postpone time. (.1 instead of 0).
         # This small change ensures that if for a split second a process
         # outputs a lot of information, we don't give the highest priority to
@@ -397,7 +430,7 @@ class Pymux(object):
         cli.max_render_postpone_time = .1  # Second.
 
         # Hide message when a key has been pressed.
-        def key_pressed():
+        def key_pressed(_):
             self.get_client_state(cli).message = None
         cli.input_processor.beforeKeyPress += key_pressed
 
@@ -409,8 +442,6 @@ class Pymux(object):
         # change size, so everything has to be redrawn.)
         self.invalidate()
 
-        cli.on_invalidate += lambda: self.invalidate()
-
         # Handle start-up comands.
         # (Does initial key bindings.)
         if not self._startup_done:
@@ -466,7 +497,7 @@ class Pymux(object):
 
         # Set session_name according to socket name.
         if '.' in self.socket_name:
-            self.session_name = self.socket_name.rsplit('.')[-1]
+            self.session_name = self.socket_name.rpartition('.')[-1]
 
         logger.info('Listening on %r.' % self.socket_name)
         return self.socket_name
@@ -493,6 +524,9 @@ class Pymux(object):
 
         signal.signal(signal.SIGINT, handle_sigint)
 
+        # Start background threads.
+        self._start_auto_refresh_thread()
+
         # Run eventloop.
 
         # XXX: Both the PipeInput and DummyCallbacks are not used.
@@ -513,18 +547,21 @@ class Pymux(object):
             os.close(fd)
             raise
 
-        # Clean up socket.
-        os.remove(self.socket_name)
+        finally:
+            # Clean up socket.
+            os.remove(self.socket_name)
 
-    def run_standalone(self, true_color=False):
+    def run_standalone(self, true_color=False, ansi_colors_only=False):
         """
         Run pymux standalone, rather than using a client/server architecture.
         This is mainly useful for debugging.
         """
         self._runs_standalone = True
+        self._start_auto_refresh_thread()
         cli = self.create_cli(
             connection=None,
-            output=Vt100_Output.from_pty(sys.stdout, true_color=true_color))
+            output=Vt100_Output.from_pty(
+                sys.stdout, true_color=true_color, ansi_colors_only=ansi_colors_only))
         cli._is_running = False
         cli.run()
 
